library(lpSolve)

dantzig=function(X,Y,delta,gamma,vlength){
  k=dim(X)[2]
  c1=matrix(1,k,1)
  c2=matrix(0,k,1)
  c=rbind(c1,c2)
  A11=t(X)%*%X
  A12=-t(X)%*%X
  A1=cbind(A11,A12)
  A21=-t(X)%*%X
  A22=t(X)%*%X
  A2=cbind(A21,A22)
  A31=2*diag(k)
  A32=-diag(k)
  A3=cbind(A31,A32)
  A=rbind(A1,A2,A3)
  b1=-t(X)%*%Y-delta*matrix(1,k,1)
  b2=t(X)%*%Y-delta*matrix(1,k,1)
  b3=matrix(0,k,1)
  b=rbind(b1,b2,b3)
  dir=matrix(">=",dim(b)[1],1)
  
  vars=lp("min", c, A, dir, b)$solution
  vars=as.vector(vars)
  u=vars[1:k]
  beta=(vars[(k+1):length(vars)]-u)/vlength
  terms=(abs(beta)>gamma)
  answer=list(sum(terms),terms,beta)
  return(answer)
}

##   Dantzig Selector tuning parameters and variable selection   ##
dantzig.selection_AICc <- function(X,Y)   {
  gamma=0.5 #change depending on preferences
  n=length(Y)
  Yc <- Y-mean(Y)
  vlength <- sqrt(diag(t(X)%*%X))
  Xs <- t(t(X)*(1/vlength))
  i=0
  d0=round(max(abs((t(Xs)%*%Yc))))
  output=matrix(0,nrow=d0+1,ncol=3)
  for (delta in seq(0,d0,by=1)){
    i=i+1
    out=dantzig(Xs,Yc,delta,gamma,vlength)
    p=out[[1]]+1
    if (sum(out[[2]])==0) Xmodel=rep(1,dim(X)[1])
    else Xmodel=X[,out[[2]]]
    lin=lm(Y~Xmodel)
    SSE=deviance(lin)
    AICc=n*log(SSE/n)+(2*p)+((2*p*(p+1))/(n-p-1))
    output[i,1]=delta
    output[i,2]=p
    output[i,3]=AICc
  }
  delta.best=(which(output[,3]==min(output[,3]))[1])-1
  model.terms=dantzig(Xs,Yc,delta.best,gamma,vlength)[[2]]
  return(model.terms)
}