DantzigSim <- function(D,iters=1000) {
#Main function to perform power simulation with the Dantzig Selector
#inputs: 
#D: design matrix (coded as +/-1)
#iters: number of iterations for each simulation scenario (default is 1000)
  
#output:
#matrix of simulation results;
#columns: run size, number of factors, number of active main effects in true model, number of active 2FI in true model, 
#         2FI size (1=Equal to main effects, 2=Smaller than main effects), power, power for main effects, power for 2FIs,
#         indivdual error rate (IER), IER for main effects, IER for 2FIs, experimentwise error rate (EER), EER for main effects
#         EER for 2FIs, false discvoery rate (FDR), FDR for main effects, FDR for 2FIs, average number of terms selected
   
  source("DantzigSelector.r")
  source("heredity.r")
  
    getmats <- heredity(D)
    allx <- getmats[[1]][,-1]
    ME=dim(D)[2]
    rel <- getmats[[2]][2:(ME+1),]
    j=0
    results <- matrix(0,nrow=56,ncol=18)
    for (intsize in c(1,2)) {
      for (num_active_ME in 2:5) {
        for (num_active_2fi in 1:7) {
          j=j+1
          num_active <- num_active_ME+num_active_2fi
          prop <- rep(0,iters) 
          propME <- rep(0,iters)
          prop2fi <- rep(0,iters)
          error <- rep(0,iters)
          errorME <- rep(0,iters)
          error2fi <- rep(0,iters)
          EER <- rep(0,iters)
          EER_ME <- rep(0,iters)
          EER_2fi <- rep(0,iters)
          falsediscovery <- rep(0,iters)
          falsediscoveryME <- rep(0,iters)
          falsediscovery2fi <- rep(0,iters)
          numterms <- rep(0,iters)
          for (i in 1:iters) {
            active_ME <- sort(sample(ME,num_active_ME))
            inactive_ME <- setdiff(1:ME,active_ME)
            H <- rel[active_ME,-1]
            cand_2fi <- which(apply(H,2,sum)!=0)
            active_2fi <- sort(sample(cand_2fi,num_active_2fi))
            inactive_2fi <- setdiff((ME+1):dim(allx)[2],active_2fi)
            active <- c(active_ME,active_2fi)
            effects <- 1:dim(allx)[2]
            true <- effects%in%active
            inactive <- which(true %in% c(0))
            if (intsize==1) acoeff <- sample(c(0.5,1,1.5,2,2.5,3,3.5),num_active,replace=TRUE)*(((round(runif(num_active)))*2)-1)
            if (intsize==2) {
              MEcoeff <- sample(c(2,2.5,3,3.5),num_active_ME,replace=TRUE)*(((round(runif(num_active_ME)))*2)-1)
              Intcoeff <- sample(c(0.5,1,1.5,2),num_active_2fi,replace=TRUE)*(((round(runif(num_active_2fi)))*2)-1)
              acoeff <- c(MEcoeff,Intcoeff)
            }
            y <- allx[,active]%*%acoeff + rnorm(dim(allx)[1])
            selected <- dantzig.selection_AICc(allx,y) 
            found <- which(selected %in% c(1))
            found_ind <- effects%in%found
            sum_ind <- true+found_ind
            foundME=sum(found%in%(1:ME))
            found2fi=sum(found%in%(ME+1):dim(allx)[2])
            numbcorrect <- active%in%found
            numbMEcorrect <- active_ME%in%found
            numb2ficorrect <- active_2fi%in%found
            typeI <- inactive%in%found
            typeIME <- inactive_ME%in%found
            typeI2fi <- inactive_2fi%in%found
            prop[i] <- sum(numbcorrect)/num_active
            propME[i] <- sum(numbMEcorrect)/num_active_ME
            prop2fi[i] <- sum(numb2ficorrect)/num_active_2fi
            error[i] <- sum(typeI)/length(typeI)
            errorME[i] <- sum(typeIME)/length(typeIME)
            error2fi[i] <- sum(typeI2fi)/length(typeI2fi)
            EER[i] <- (sum(typeI)>=1)
            EER_ME[i] <- (sum(typeIME)>=1)
            EER_2fi[i] <- (sum(typeI2fi)>=1)
            numterms[i] <- length(found)
            if (length(found)==0) {
              falsediscovery[i]=0
              falsediscoveryME[i]=0
              falsediscovery2fi[i]=0 
            }
            else {
              falsediscovery[i] <- sum(typeI)/length(found)
              if (foundME==0) falsediscoveryME[i]=0
              else falsediscoveryME[i] <- sum(typeIME)/foundME
              if (found2fi==0) falsediscovery2fi[i]=0
              else falsediscovery2fi[i]=sum(typeI2fi)/found2fi
            }
          }  
          power <- mean(prop)
          powerME <- mean(propME)
          power2fi <- mean(prop2fi)
          typeIerror <- mean(error)
          typeIerrorME <- mean(errorME)
          typeIerror2fi <- mean(error2fi)
          EERtypeI <- sum(EER)/iters
          EERtypeIME <- sum(EER_ME)/iters
          EERtypeI2fi <- sum(EER_2fi)/iters
          FDR <- mean(falsediscovery)
          FDRME <- mean(falsediscoveryME)
          FDR2fi <- mean(falsediscovery2fi)
          avgnumterms <- mean(numterms)
          results[j,] <- c(nrow(D),ncol(D), num_active_ME,num_active_2fi,intsize, 
                           power,powerME,power2fi,typeIerror,typeIerrorME,typeIerror2fi,EERtypeI,EERtypeIME,EERtypeI2fi,FDR,FDRME,FDR2fi,avgnumterms) 
          
        }      
     }
  }
  results <- as.data.frame(results)
  names(results) <- c("n","factors","Num Active ME","Num Active 2FI","2FI Size","Power", "Power ME", "Power 2FI","IER", "IER ME",
                      "IER 2FI","EER", "EER ME", "EER 2FI", "FDR", "FDR ME", "FDR 2FI", "Num Terms Selected")
  return(results)
}
  
 

