function results = ForwardSim(D,iters)

%Main function to perform power simulation with Forward Selection assuming
%weak effect heredity

%inputs: 
%D: design matrix (coded as +/-1)
%iters: number of iterations for each simulation scenario (default is 1000)
  
%output:
%matrix of simulation results;
%columns: run size, number of factors, number of active main effects in true model, number of active 2FI in true model, 
%         2FI size (1=Equal to main effects, 2=Smaller than main effects), power, power for main effects, power for 2FIs,
%         indivdual error rate (IER), IER for main effects, IER for 2FIs, experimentwise error rate (EER), EER for main effects
%         EER for 2FIs, false discvoery rate (FDR), FDR for main effects, FDR for 2FIs, average number of terms selected

  results=[];
  switch nargin
      case 1
          iters=1000;
  end
  [M, relation, labels]=fullmatrix(D); %create relationship matrix
  allx=M(:,2:end); %full model matrix (minus the intercept column)
  ME=size(D,2); %number of main effects
  rel=relation(2:(ME+1),:); %subset of relationship matrix for weak effect heredity
  for intsize = [1 2]
    for num_active_ME=[2 3 4 5]
     for num_active_2fi=[1 2 3 4 5 6 7]
          num_active=num_active_ME+num_active_2fi;
          prop=zeros(iters,1);
          propME=zeros(iters,1);
          prop2fi=zeros(iters,1);
          error=zeros(iters,1);
          errorME=zeros(iters,1);
          error2fi=zeros(iters,1);
          EER=zeros(iters,1);
          EERME=zeros(iters,1);
          EER2fi=zeros(iters,1);
          falsediscovery=zeros(iters,1);
          falsediscoveryME=zeros(iters,1);
          falsediscovery2fi=zeros(iters,1);
          numterms=zeros(iters,1);
          
          for i=1:iters
              active_ME=sort(randsample(ME,num_active_ME));
              inactive_ME=setdiff(1:ME,active_ME);
              heredity=rel(active_ME,2:end);
              cand_2fi=find(sum(heredity)~=0);
              active_2fi=sort(randsample(cand_2fi,num_active_2fi));
              inactive_2fi=setdiff((ME+1):size(allx,2),active_2fi);
              active=sort([active_ME' active_2fi]);
              true=ismember(1:size(allx,2),active);
              inactive=find(true==0);
              if intsize==1
              	acoeff=randsample([0.5 1 1.5 2 2.5 3 3.5],num_active,'true')'.*((round(rand(num_active,1))*2)-1);
              elseif intsize==2
                MEcoeff=randsample([2 2.5 3 3.5],num_active_ME,'true')'.*((round(rand(num_active_ME,1))*2)-1);
                Intcoeff=randsample([0.5 1 1.5 2],num_active_2fi,'true')'.*((round(rand(num_active_2fi,1))*2)-1);
                acoeff=[MEcoeff; Intcoeff];
              end
              
              y=allx(:,active)*acoeff+randn(size(allx,1),1);
              inmodel=forward_selection(allx,y,ME,rel,0.5);
                                  
              found=find(inmodel==1);
              foundME=sum(ismember(found,1:ME));
              found2fi=sum(ismember(found,(ME+1):size(allx,2)));
              numbcorrect=ismember(active,found);
              numbMEcorrect=ismember(active_ME,found);
              numb2ficorrect=ismember(active_2fi,found);
              typeI=ismember(inactive, found);
              typeIME=ismember(inactive_ME,found);
              typeI2fi=ismember(inactive_2fi,found);
              prop(i)=sum(numbcorrect)/num_active;
              propME(i)=sum(numbMEcorrect)/num_active_ME;
              prop2fi(i)=sum(numb2ficorrect)/num_active_2fi;
              error(i)=sum(typeI)/size(typeI,2);
              errorME(i)=sum(typeIME)/size(typeIME,2);
              error2fi(i)=sum(typeI2fi)/size(typeI2fi,2);
              EER(i)=(sum(typeI)>=1);
              EERME(i)=(sum(typeIME)>=1);
              EER2fi(i)=(sum(typeI2fi)>=1);
              numterms(i)=length(found);
              if isempty(found)
		        falsediscovery(i)=0;
                falsediscoveryME(i)=0;
                falsediscovery2fi(i)=0;
              else
                falsediscovery(i)=sum(typeI)/length(found);
                falsediscoveryME(i)=sum(typeIME)/foundME;
                if found2fi==0
                    falsediscovery2fi(i)=0;
                else
                    falsediscovery2fi(i)=sum(typeI2fi)/found2fi;
                end
              end
          end
                
          power=mean(prop);
          powerME=mean(propME);
          power2fi=mean(prop2fi);
          typeIerror=mean(error);
          typeIerrorME=mean(errorME);
          typeIerror2fi=mean(error2fi);
          EERtypeI=sum(EER)/iters;
          EERtypeIME=sum(EERME)/iters;
          EERtypeI2fi=sum(EER2fi)/iters;
          FDR=mean(falsediscovery);
          FDRME=mean(falsediscoveryME);
          FDR2fi=mean(falsediscovery2fi);
          avgnumterms=mean(numterms);
          results=[results; intsize num_active_ME num_active_2fi power powerME power2fi typeIerror typeIerrorME typeIerror2fi EERtypeI EERtypeIME EERtypeI2fi FDR FDRME FDR2fi avgnumterms];
     end
 
    end
  end

